<?php

namespace FiloBlu\Flow\Model\Actions\From;

/**
 * Class Addincrementstock
 *
 * @package FiloBlu\Flow\Model\Actions\From
 */
class Addincrementstock
{

    /**
     * @var \FiloBlu\Flow\Helper\Product
     */
    protected $_helperProduct;

    /**
     * @var \Magento\Catalog\Model\ProductFactory
     */
    protected $_productFactory;

    /**
     * @var \FiloBlu\Flow\Helper\LoggerProvider
     */
    protected $_logger;

    /**
     * @var \Magento\CatalogInventory\Api\StockRegistryInterface
     */
    protected $_stockRegistry;

    /**
     * @var \FiloBlu\Flow\Helper\Stock
     */
    protected $_stockQty;

    /**
     * @var \Magento\Framework\App\ProductMetadataInterface
     */
    protected $_productMetadata;

    /**
     * Addincrementstock constructor.
     *
     * @param \FiloBlu\Flow\Helper\Product                         $helperProduct
     * @param \FiloBlu\Flow\Helper\LoggerProvider                  $loggerProvider
     * @param \Magento\Catalog\Model\ProductFactory                $productFactory
     * @param \Magento\CatalogInventory\Api\StockRegistryInterface $stockRegistry
     * @param \FiloBlu\Flow\Helper\Stock                           $stockQty
     * @param \Magento\Framework\App\ProductMetadataInterface      $productMetadata
     * @param array                                                $data
     */
    public function __construct(
        \FiloBlu\Flow\Helper\Product $helperProduct,
        \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider,
        \Magento\Catalog\Model\ProductFactory $productFactory,
        \Magento\CatalogInventory\Api\StockRegistryInterface $stockRegistry,
        \FiloBlu\Flow\Helper\Stock $stockQty,
        \Magento\Framework\App\ProductMetadataInterface $productMetadata,
        array $data = []
    ) {
        $this->_helperProduct = $helperProduct;
        $this->_productFactory = $productFactory;
        $this->_logger = $loggerProvider->getLogger();
        $this->_stockRegistry = $stockRegistry;
        $this->_stockQty = $stockQty;
        $this->_productMetadata = $productMetadata;
    }

    /**
     * @param $product
     * @param $value
     * @param $storeId
     */
    public function process($product, $value, $storeId)
    {
        \Magento\Framework\App\ObjectManager::getInstance()->get('Magento\Store\Model\StoreManagerInterface')->setCurrentStore($storeId);
        if ($storeId != 0) {
            return "The function is avalaible only for the ADMIN store.";
        }

        $productId = $product->getId();

        if(empty($productId)){
            $message = sprintf("Product not found! product_code: '%s'", $product->getSku());
            $this->_logger->error(false, $message);
            return false;
        }
        else {
            if($value != null && $value != ''){
                $stock = $this->_stockRegistry->getStockItem($productId);

                if(is_numeric($value) && $value > 0){
                    $stock->setUseConfigEnableQtyInc(false);
                    $stock->setUseConfigQtyIncrements(false);
                    $stock->setEnableQtyIncrements(true);
                    $stock->setQtyIncrements($value);
                    $stock->save();
                }
                else if(is_numeric($value) && $value < 0) {
                    $stock->setEnableQtyIncrements(false);
                    $stock->setUseConfigEnableQtyInc(true);
                    $stock->setUseConfigQtyIncrements(true);
                    $stock->save();
                }
                else {
                    $message = sprintf("Product increment value wrong: '%s'", $value);
                    $this->_logger->error(false, $message);
                    return false;
                }
            }
        }
    }

}
