<?php
namespace FiloBlu\Flow\Model\Channel\In;

/**
 * Class AbstractModel
 * @package FiloBlu\Flow\Model\Channel\In
 */
abstract class AbstractModel {

    /**
     * @var
     */
    protected $_name;
    /**
     * @var
     */
    protected $_data;
    /**
     * @var \FiloBlu\Flow\Model\Connector\AbstractConnector
     */
    protected $_connector;
    /**
     * @var
     */
    protected $_parser;
    /**
     * @var \FiloBlu\Flow\Model\Map $map
     */
    public $map;
    protected $_config;
    /**
     * @var \FiloBlu\Flow\Model\Channel\Config
     */
    protected $_configModel;
    protected $_file;
    protected $_insertQueue =[];
    protected $_tablename;
    /**
    * @var \Monolog\Logger
    */
    protected $_logger;
    /**
     * @var  \Magento\Framework\App\ObjectManager;
     */
    protected $_objectManager;
    /**
     * AbstractModel constructor.
     * @param \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider
     * @param \FiloBlu\Flow\Model\Channel\ConfigFactory $channelConfigFactory
     */
    public function __construct(
        \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider,
        \FiloBlu\Flow\Model\Channel\ConfigFactory $channelConfigFactory
    ) {
        $this->_logger = $loggerProvider->getLogger();
        $this->_configModel = $channelConfigFactory->create();
        $this->_objectManager = \Magento\Framework\App\ObjectManager::getInstance();
    }

    /**
     * @param $channelname
     */
    public function init($channelname) {
        $this->loadConfig($channelname);
    }

    /**
     * @param $data
     * @return bool
     */
    public function insertData($data) {

        // Genero il FLusso ITEMS
        $model =  \Magento\Framework\App\ObjectManager::getInstance()->create("FiloBlu\Flow\Model\\" . $this->getConfig('flow'));

        if (!$model) {
            return false;
        }

        $model->setData('meta_file',$this->getFile()->getId());

        foreach($data as $field => $value){
            $fieldMap = $this->map->get($field);
            if(!isset($fieldMap["skip"])) {
                $to_field = $fieldMap["to"];
                $model->setData($to_field,$value);
            }
        }

        $model->save();

    }

    /**
     * @param $name
     */
    public function setName($name) {
        $this->_name = $name;
    }

    /**
     * @return mixed
     */
    public function getName() {
        return $this->_name;
    }

    /**
     * @param $key
     * @return null
     */
    public function getConfig($key) {
        if(isset($this->_config[$key])) {
            return $this->_config[$key];
        }
        return null;
    }

    /**
     * @param $key
     * @return null
     */
    public function getData($key) {
        if(isset($this->_data[$key])) {
            return $this->_data[$key];
        }
        return null;
    }

    /**
     * @param $config
     */
    public function load($config) {
        $this->_data = (Array) $config;
    }

    /**
     *
     */
    public function storeData() {

    }

    /**
     * @param $connector
     * @throws \Exception
     */
    public function setConnector($connector) {

        $connector_type = $connector['type'];

        $connector_class = \Magento\Framework\App\ObjectManager::getInstance()->create("FiloBlu\Flow\Model\Connector\\" . ucfirst($connector_type));

        if (trim($connector_type) and ($connector_class instanceof \FiloBlu\Flow\Model\Connector\AbstractConnector)) {
            $this->_connector = $connector_class;
            $this->_connector->setConfig($connector);
        }
        else {
            throw new \Exception('Error in connector config');
        }
    }

    /**
     * @return \FiloBlu\Flow\Model\Connector\AbstractConnector
     */
    public function getConnector() {
        return $this->_connector;
    }

    /**
     * @param $parser
     * @throws \Exception
     */
    public function setParser($parser) {

        $parser_type = $parser['type'];

        $parser_class = \Magento\Framework\App\ObjectManager::getInstance()->create("FiloBlu\Flow\Model\Parser\\" . ucfirst($parser_type));

        if (trim($parser_type) and ($parser_class instanceof \FiloBlu\Flow\Model\Parser\AbstractParser)) {
            $this->_parser = $parser_class;
            $this->_parser->setConfig($parser);
        }

        else {
            throw new \Exception('Error in parser config');
        }
    }

    /**
     * @return mixed
     */
    public function getParser() {
        return $this->_parser;
    }

    /**
     *
     */
    public function processBeginningOfFlow() {

    }

    /**
     *
     */
    public function processEndOfFlow() {

    }

    /**
     * @return bool
     * @throws \Exception
     */
    public function rlist(){

        if (!$this->_connector) {
            throw new \Exception('no connector joined to channel');
        }

        if($this->getData("depends_folder")) {
            $depends_resources = $this->_connector->listDir($this->getData("depends_folder"));
            if(count($depends_resources) > 0 ) {
                return false;
            }
        }

        if($this->getData("depends_file")){
            $depends_resources = $this->_connector->getResources();;
            if(count($depends_resources) > 0 ){
                return false;
            }
        }

        $resources = $this->_connector->getResources();

        if (!count($resources)) {
            return false;
        }

        return $resources;

    }

    /**
     * @param $file
     * @return mixed
     * @throws \Exception
     */
    public function receiveFile($file){

        if (!$this->_connector) {
            throw new \Exception('No connector joined to channel');
        }

        return $this->_connector->receiveResource($file->getName());
    }

    /**
     * @param $file
     * @return bool
     * @throws \Exception
     */
    public function parse($file) {

        if (!$this->_connector) {
            throw new \Exception('no connector joined to channel');
        }

        if (!$this->_parser) {
            throw new \Exception('no parser joined to channel');
        }

        $this->setFile($file);
        $localdir = $this->_connector->getLocalDir();

        $this->_parser->loadMap($file->getChannel());
        $this->setMap($this->_parser->map);
        $this->_parser->setChannel($this);

        //Il Parser chiama il channel con il metodo insertData
        try {
            $this->_parser->process($localdir . "/" . $file->getName());
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
        return true;

    }


    public function setFile($file){
        $this->_file = $file;
        return $this;
    }


    public function getFile(){
        return $this->_file;
    }


    public function processendingofFlow() {

    }

    /**
     * @param \FiloBlu\Flow\Model\Map $map
     *
     * @return $this
     */
    public function setMap($map){
        $this->map = $map;
        return $this;
    }

    /**
     * TODO: Remove
     * @deprecatd
     * @return string
     */
    protected function _getWebsitesIds(){
        if(!isset($this->websites)) {
            $websitesIds = array();
            $websites = Mage::app()->getWebsites();
            foreach ($websites as $website) {
                $websitesIds[] = $website->getId();
            }
            $this->websites = implode(",",$websitesIds);
        }
        return $this->websites;
    }


    protected function _getStoreIdByCode($code){
        if(!isset($this->stores)){
            $stores= Mage::app()->getStores();
            $stores_by_code = array();
            foreach($stores as $store){
                $stores_by_code[$store->getCode()] = $store->getId();
            }
            $this->stores = $stores_by_code;
        }
        return $this->stores[$code];
    }


    public function commitData(){
        return true;
    }


    public function loadConfig($name){

        // $this->_logger->info($name);

        // $name -> DB flow_channels.name
        // unique value (string)

        $this->_configModel->load($name);

        if($this->_configModel->initialized){
            $this->_config = $this->_configModel->get('config');
            $this->setConnector($this->_configModel->get('connector'));
            $this->setParser($this->_configModel->get('parser'));
            return true;
        }
        return false;
    }

}
